==> xdatabase-1.0.6.tar.gz: lib/src/table_meta.dart <==
class FieldDeclaration {
  late Symbol symbol;
  late String fieldName;
  late String fieldType;
  late bool primary;
  late String mark;
  FieldDeclaration(
      {required this.symbol,
      required this.fieldName,
      required this.fieldType,
      required this.primary,
      required this.mark});

  FieldDeclaration.fromMap(Map map) {
    fieldName = map['fieldName'] as String;
    fieldType = map['fieldType'] as String;
    primary = map['primary'] as bool;
    mark = map['mark'] as String;
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{
      'fieldName': fieldName,
      'fieldType': fieldType,
      'primary': primary,
      'mark': mark
    };
  }

  String createScript() {
    if (primary) {
      if (fieldType.toLowerCase() == 'int') {
        return '$fieldName serial PRIMARY KEY';
      } else {
        return '$fieldName $fieldType PRIMARY KEY';
      }
    } else {
      return '$fieldName $fieldType';
    }
  }
}

class IndexDeclaration {
  late String indexName;
  late List<String> fields;
  IndexDeclaration({required this.indexName, required this.fields});

  IndexDeclaration.fromMap(Map map) {
    indexName = map['indexName'] as String;
    fields = map['fields'].cast<String>();
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{'indexName': indexName, 'fields': fields};
  }
}

class UniqueDeclaration {
  late String indexName;
  late List<String> fields;
  UniqueDeclaration({required this.indexName, required this.fields});

  UniqueDeclaration.fromMap(Map map) {
    indexName = map['indexName'] as String;
    fields = map['fields'].cast<String>();
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{'indexName': indexName, 'fields': fields};
  }
}

class TableDeclaration {
  late String tableName;
  late List<FieldDeclaration> fields;
  late List<IndexDeclaration> indexes;
  late List<UniqueDeclaration> uniqueIndexes;
  TableDeclaration({required this.tableName})
      : fields = <FieldDeclaration>[],
        indexes = <IndexDeclaration>[],
        uniqueIndexes = <UniqueDeclaration>[];

  TableDeclaration.fromMap(Map map) {
    print('map table: $map');
    tableName = map['tableName'] as String;
    fields = map['fields']
        .map((e) => FieldDeclaration.fromMap(e))
        .toList()
        .cast<FieldDeclaration>();
    indexes = map['indexes']
        .map((e) => IndexDeclaration.fromMap(e))
        .toList()
        .cast<IndexDeclaration>();
    uniqueIndexes = map['uniqueIndexes']
        .map((e) => UniqueDeclaration.fromMap(e))
        .toList()
        .cast<UniqueDeclaration>();
  }

  Map<String, dynamic> asMap() {
    return {
      'tableName': tableName,
      'fields': fields.map((e) => e.asMap()).toList(),
      'indexes': indexes.map((e) => e.asMap()).toList(),
      'uniqueIndexes': uniqueIndexes.map((e) => e.asMap()).toList()
    };
  }

  String createScript() {
    var lines = [
      'CREATE TABLE IF NOT EXISTS $tableName',
      '(',
      fields.map((e) => e.createScript()).toList().join(',\n'),
      ');'
    ];
    lines.add('\n');
    lines.addAll(indexes.map((e) => createIndexSQL(e)).toList());
    lines.add('\n');
    lines.addAll(uniqueIndexes.map((e) => createUniqueSQL(e)).toList());
    lines.add('\n');
    return lines.join('\n');
  }

  String compareWith(TableDeclaration otherTable) {
    var lines = [];
    final fieldsNeedsToAdd = fields
        .where((e1) => otherTable.fields
            .where((e2) => e2.fieldName == e1.fieldName)
            .toList()
            .isEmpty)
        .toList();
    // TODO: 加入需要新增的列
    lines.addAll(fieldsNeedsToAdd.map((e) => alterTableAddColumn(e)).toList());

    final fieldsNeedsToRemove = otherTable.fields
        .where((e1) =>
            fields.where((e2) => e2.fieldName == e1.fieldName).toList().isEmpty)
        .toList();
    // TODO: 加入需要删除的列
    lines.addAll(
        fieldsNeedsToRemove.map((e) => alterTableDropColumn(e)).toList());

    final indexNeedToAdd = indexes
        .where((idx) => otherTable.indexes
            .where((cidx) => cidx.indexName == idx.indexName)
            .toList()
            .isEmpty)
        .toList();
    final indexNeedToDelete = otherTable.indexes
        .where((idx) => indexes
            .where((nidx) => nidx.indexName == idx.indexName)
            .toList()
            .isEmpty)
        .toList();

    lines.addAll(indexNeedToAdd.map((e) => createIndexSQL(e)).toList());
    lines.addAll(indexNeedToDelete.map((e) => dropIndexSQL(e)).toList());

    final uniqueNeedToAdd = uniqueIndexes
        .where((u1) => otherTable.uniqueIndexes
            .where((u2) => u1.indexName == u2.indexName)
            .toList()
            .isEmpty)
        .toList();

    final uniqueNeedToDelete = otherTable.uniqueIndexes
        .where((u1) => uniqueIndexes
            .where((u2) => u1.indexName == u2.indexName)
            .toList()
            .isEmpty)
        .toList();

    lines.addAll(uniqueNeedToAdd.map((e) => createUniqueSQL(e)).toList());
    lines.addAll(uniqueNeedToDelete.map((e) => dropUniqueSQL(e)).toList());

    return lines.join('\n\n');
  }

  String alterTableAddColumn(FieldDeclaration field) {
    return 'ALTER TABLE "$tableName" ADD COLUMN "${field.fieldName}" ${field.fieldType};';
  }

  String alterTableDropColumn(FieldDeclaration field) {
    return 'ALTER TABLE "$tableName" DROP COLUMN "${field.fieldName}";';
  }

  String alterTableModify(FieldDeclaration current, FieldDeclaration latest) {
    if (latest.fieldType.toLowerCase().startsWith('varchar') &&
        current.fieldType.toLowerCase().startsWith('varchar')) {
      return 'ALTER TABLE "$tableName" ALTER COLUMN "${current.fieldName}" TYPE "${current.fieldType}";';
    } else {
      var commands = [];
      commands.add(alterTableDropColumn(current));
      commands.add(alterTableAddColumn(current));
      return commands.join('\n');
    }
  }

  String dropTableSql() {
    return 'DROP TABLE IF EXISTS "$tableName";';
  }

  String createIndexSQL(IndexDeclaration idx) {
    return 'CREATE INDEX ${idx.indexName} ON $tableName (${idx.fields.map((fn) => [
          '"',
          fn,
          '"'
        ].join('')).toList().join(',')});';
  }

  String dropIndexSQL(IndexDeclaration idx) {
    return 'DROP INDEX "${idx.indexName}";';
  }

  String createUniqueSQL(UniqueDeclaration idx) {
    return 'CREATE UNIQUE INDEX ${idx.indexName} ON $tableName (${idx.fields.map((fn) => [
          '"',
          fn,
          '"'
        ].join('')).toList().join(',')});';
  }

  String dropUniqueSQL(UniqueDeclaration idx) {
    return 'DROP UNIQUE INDEX "${idx.indexName}";';
  }
}

class Declarations {
  late Map<String, TableDeclaration> tables;
  Declarations() : tables = <String, TableDeclaration>{};

  Declarations.fromMap(Map map) {
    tables =
        map.map((key, value) => MapEntry(key, TableDeclaration.fromMap(value)));
  }

  Map<String, dynamic> asMap() {
    return tables.map((key, value) => MapEntry(key, value.asMap()));
  }

  String createTables() {
    final tableScripts = tables.values.map((e) => e.createScript());
    return tableScripts.join('\n\n');
  }
}

==> xdatabase-1.0.8.tar.gz: lib/src/table_meta.dart <==
class FieldDeclaration {
  late Symbol symbol;
  late String propertyName;
  late String fieldName;
  late String fieldType;
  late bool primary;
  late String mark;
  FieldDeclaration(
      {required this.symbol,
      required this.propertyName,
      required this.fieldName,
      required this.fieldType,
      required this.primary,
      required this.mark});

  FieldDeclaration.fromMap(Map map) {
    propertyName = map['propertyName'] as String;
    fieldName = map['fieldName'] as String;
    fieldType = map['fieldType'] as String;
    primary = map['primary'] as bool;
    mark = map['mark'] as String;
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{
      'propertyName': propertyName,
      'fieldName': fieldName,
      'fieldType': fieldType,
      'primary': primary,
      'mark': mark
    };
  }

  String createScript() {
    if (primary) {
      if (fieldType.toLowerCase() == 'int') {
        return '$fieldName serial PRIMARY KEY';
      } else {
        return '$fieldName $fieldType PRIMARY KEY';
      }
    } else {
      return '$fieldName $fieldType';
    }
  }
}

class IndexDeclaration {
  late String indexName;
  late List<String> fields;
  IndexDeclaration({required this.indexName, required this.fields});

  IndexDeclaration.fromMap(Map map) {
    indexName = map['indexName'] as String;
    fields = map['fields'].cast<String>();
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{'indexName': indexName, 'fields': fields};
  }
}

class UniqueDeclaration {
  late String indexName;
  late List<String> fields;
  UniqueDeclaration({required this.indexName, required this.fields});

  UniqueDeclaration.fromMap(Map map) {
    indexName = map['indexName'] as String;
    fields = map['fields'].cast<String>();
  }

  Map<String, dynamic> asMap() {
    return <String, dynamic>{'indexName': indexName, 'fields': fields};
  }
}

class TableDeclaration {
  late String tableName;
  late String className;
  late List<FieldDeclaration> fields;
  late List<IndexDeclaration> indexes;
  late List<UniqueDeclaration> uniqueIndexes;
  TableDeclaration({required this.tableName, required this.className})
      : fields = <FieldDeclaration>[],
        indexes = <IndexDeclaration>[],
        uniqueIndexes = <UniqueDeclaration>[];

  TableDeclaration.fromMap(Map map) {
    tableName = map['tableName'] as String;
    className = map['className'] as String;
    fields = map['fields']
        .map((e) => FieldDeclaration.fromMap(e))
        .toList()
        .cast<FieldDeclaration>();
    indexes = map['indexes']
        .map((e) => IndexDeclaration.fromMap(e))
        .toList()
        .cast<IndexDeclaration>();
    uniqueIndexes = map['uniqueIndexes']
        .map((e) => UniqueDeclaration.fromMap(e))
        .toList()
        .cast<UniqueDeclaration>();
  }

  Map<String, dynamic> asMap() {
    return {
      'tableName': tableName,
      'className': className,
      'fields': fields.map((e) => e.asMap()).toList(),
      'indexes': indexes.map((e) => e.asMap()).toList(),
      'uniqueIndexes': uniqueIndexes.map((e) => e.asMap()).toList()
    };
  }

  String createScript() {
    var lines = [
      'CREATE TABLE IF NOT EXISTS $tableName',
      '(',
      fields.map((e) => e.createScript()).toList().join(',\n'),
      ');'
    ];
    lines.add('\n');
    lines.addAll(indexes.map((e) => createIndexSQL(e)).toList());
    lines.add('\n');
    lines.addAll(uniqueIndexes.map((e) => createUniqueSQL(e)).toList());
    lines.add('\n');
    return lines.join('\n');
  }

  String compareWith(TableDeclaration otherTable) {
    var lines = [];
    final fieldsNeedsToAdd = fields
        .where((e1) => otherTable.fields
            .where((e2) => e2.fieldName == e1.fieldName)
            .toList()
            .isEmpty)
        .toList();
    // TODO: 加入需要新增的列
    lines.addAll(fieldsNeedsToAdd.map((e) => alterTableAddColumn(e)).toList());

    final fieldsNeedsToRemove = otherTable.fields
        .where((e1) =>
            fields.where((e2) => e2.fieldName == e1.fieldName).toList().isEmpty)
        .toList();
    // TODO: 加入需要删除的列
    lines.addAll(
        fieldsNeedsToRemove.map((e) => alterTableDropColumn(e)).toList());

    final indexNeedToAdd = indexes
        .where((idx) => otherTable.indexes
            .where((cidx) => cidx.indexName == idx.indexName)
            .toList()
            .isEmpty)
        .toList();
    final indexNeedToDelete = otherTable.indexes
        .where((idx) => indexes
            .where((nidx) => nidx.indexName == idx.indexName)
            .toList()
            .isEmpty)
        .toList();

    lines.addAll(indexNeedToAdd.map((e) => createIndexSQL(e)).toList());
    lines.addAll(indexNeedToDelete.map((e) => dropIndexSQL(e)).toList());

    final uniqueNeedToAdd = uniqueIndexes
        .where((u1) => otherTable.uniqueIndexes
            .where((u2) => u1.indexName == u2.indexName)
            .toList()
            .isEmpty)
        .toList();

    final uniqueNeedToDelete = otherTable.uniqueIndexes
        .where((u1) => uniqueIndexes
            .where((u2) => u1.indexName == u2.indexName)
            .toList()
            .isEmpty)
        .toList();

    lines.addAll(uniqueNeedToAdd.map((e) => createUniqueSQL(e)).toList());
    lines.addAll(uniqueNeedToDelete.map((e) => dropUniqueSQL(e)).toList());

    return lines.join('\n\n');
  }

  String alterTableAddColumn(FieldDeclaration field) {
    return 'ALTER TABLE "$tableName" ADD COLUMN "${field.fieldName}" ${field.fieldType};';
  }

  String alterTableDropColumn(FieldDeclaration field) {
    return 'ALTER TABLE "$tableName" DROP COLUMN "${field.fieldName}";';
  }

  String alterTableModify(FieldDeclaration current, FieldDeclaration latest) {
    if (latest.fieldType.toLowerCase().startsWith('varchar') &&
        current.fieldType.toLowerCase().startsWith('varchar')) {
      return 'ALTER TABLE "$tableName" ALTER COLUMN "${current.fieldName}" TYPE "${current.fieldType}";';
    } else {
      var commands = [];
      commands.add(alterTableDropColumn(current));
      commands.add(alterTableAddColumn(current));
      return commands.join('\n');
    }
  }

  String dropTableSql() {
    return 'DROP TABLE IF EXISTS "$tableName";';
  }

  String createIndexSQL(IndexDeclaration idx) {
    return 'CREATE INDEX ${idx.indexName} ON $tableName (${idx.fields.map((fn) => [
          '"',
          fn,
          '"'
        ].join('')).toList().join(',')});';
  }

  String dropIndexSQL(IndexDeclaration idx) {
    return 'DROP INDEX "${idx.indexName}";';
  }

  String createUniqueSQL(UniqueDeclaration idx) {
    return 'CREATE UNIQUE INDEX ${idx.indexName} ON $tableName (${idx.fields.map((fn) => [
          '"',
          fn,
          '"'
        ].join('')).toList().join(',')});';
  }

  String dropUniqueSQL(UniqueDeclaration idx) {
    return 'DROP UNIQUE INDEX "${idx.indexName}";';
  }
}

class Declarations {
  late Map<String, TableDeclaration> tables;
  Declarations() : tables = <String, TableDeclaration>{};

  Declarations.fromMap(Map map) {
    tables =
        map.map((key, value) => MapEntry(key, TableDeclaration.fromMap(value)));
  }

  Map<String, dynamic> asMap() {
    return tables.map((key, value) => MapEntry(key, value.asMap()));
  }

  String createTables() {
    final tableScripts = tables.values.map((e) => e.createScript());
    return tableScripts.join('\n\n');
  }
}
